/*
 * This file is part of the Simutrans project under the Artistic License.
 * (see LICENSE.txt)
 */

#ifndef DESCRIPTOR_WRITER_GET_WAYTYPE_H
#define DESCRIPTOR_WRITER_GET_WAYTYPE_H


#include "../../simtypes.h"


/**
 * Convert waytype string to enum waytype_t
 */
waytype_t get_waytype(char const* waytype);


#endif
