/*
 * This file is part of the Simutrans project under the Artistic License.
 * (see LICENSE.txt)
 */

#ifndef CONVOIHANDLE_H
#define CONVOIHANDLE_H


#include "tpl/quickstone_tpl.h"

class convoi_t;

typedef quickstone_tpl<convoi_t> convoihandle_t;

#endif
