// Generated by "get_pak.sh -generate_h". DO NOT EDIT.
#define PAKSET_COUNT 20
#define OBSOLETE_FROM (12)
paksetinfo_t pakinfo[PAKSET_COUNT] = {
	{ "http://downloads.sourceforge.net/project/simutrans/pak64/124-3/simupak64-124-3.zip", "pak", "pak64 124.3 r2182", 16177 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak128/pak128%20for%20ST%20124.3up%20%282.10.0%29/simupak128-2-10-for124-3up.zip", "pak128", "pak128 2.10.0 for 124.3 git r hash ", 415604 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak192.comic/pak192.comic%20V0.7.2/pak192-comic.zip", "pak192.comic", "Pak192.Comic V0.7.2 Rev 1296", 909748 },
	{ "http://simutrans-germany.com/pak.german/pak64.german_0-124-0-0-5_full.zip", "pak64.german", "pak64.german 0.124.0.0.5", 28660 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak64.japan/123-0/simupak64.japan-123-0.zip", "pak.japan", "pak64.japan 123 r2101", 10175 },
	{ "https://github.com/wa-st/pak-nippon/releases/download/v0.6.2/pak.nippon-v0.6.2.zip", "pak.nippon", "pak.nippon v0.6.2", 50198 },
	{ "http://downloads.sourceforge.net/project/simutrans/Pak128.CS/nightly%20builds/pak128.CS-r2096.zip", "pak128.CS", "Pak128.CS 0.3.0 r2096", 78628 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak128.britain/pak128.Britain%20for%20120-3/pak128.Britain.1.18-120-3.zip", "pak128.Britain", "pak128.Britain 1.18 120.3 r1991", 241715 },
	{ "http://pak128-german.de/PAK128.german_2.3_beta.zip", "PAK128.german", "Pak128.german VS 2.3.beta (Rev. 527)", 571188 },
	{ "https://github.com/Varkalandar/pak144.Excentrique/releases/download/r0.08/pak144.Excentrique_v008.zip", "pak144.Excentrique", "pak144.Excentrique v0.08", 7621 },
	{ "http://downloads.sourceforge.net/project/simutrans/pakTTD/simupakTTD-124-0.zip", "pakTTD", "pak64 120.4.1 r2146", 1995 },
	{ "http://codeberg.org/Nazalassa/pak48.bitlit/releases/download/0.1d/pak48.bitlit_0.1d.zip", "pak48.bitlit", "pak48.bitlit v0.1d", 1762 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak96.comic/pak96.comic%20for%20111-3/pak96.comic-0.4.10-plus.zip", "pak96.comic", "pak96.comic V4.1 plus", 32526 },
	{ "http://pak128.jpn.org/souko/pak128.japan.120.0.cab", "pak128.japan", "Pak128.Japan 120.0", 27780 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak32.comic/pak32.comic%20for%20102-0/pak32.comic_102-0.zip", "pak32", "", 3420 },
	{ "https://github.com/Varkalandar/pak48.Excentrique/releases/download/v0.19_RC3/pak48.excentrique_v019rc3.zip", "pak48.Excentrique", "pak48.Excentrique v0.19", 2016 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak64.contrast/pak64.Contrast_910.zip", "pakcontrast", "", 1440 },
	{ "http://downloads.sourceforge.net/project/simutrans/OldFiles/PakHD_v04B_100-0.zip", "pakHD", "Martin", 12511 },
	{ "http://downloads.sourceforge.net/project/simutrans/pakHAJO/pakHAJO_102-2-2/pakHAJO_0-102-2-2.zip", "pakHAJO", "", 7460 },
	{ "http://downloads.sourceforge.net/project/simutrans/pak64.scifi/pak64.scifi_112.x_v0.2.zip", "pak64.scifi", "pak64.SciFi V0.2", 3293 },
};
