/*
 * This file is part of the Simutrans project under the Artistic License.
 * (see LICENSE.txt)
 */

#ifndef DISPLAY_SIMIMG_H
#define DISPLAY_SIMIMG_H


#include "../simtypes.h"

/*
 * Defines to handle images
 */

typedef uint32 image_id;

#define IMG_EMPTY ((image_id)0xFFFFFFFFu)

#endif
