/*
 * This file is part of the Simutrans project under the Artistic License.
 * (see LICENSE.txt)
 */

#ifndef HALTHANDLE_H
#define HALTHANDLE_H


#include "tpl/quickstone_tpl.h"

class haltestelle_t;

typedef quickstone_tpl<haltestelle_t> halthandle_t;

#endif
